import io
import yaml
import json
import boto3
import greenstalk

import threatingestor.config
import threatingestor.exceptions

class QueueWorker:
    """Abstract base class for Queue Workers.

    Override do_work to implement child classes.
    """
    def __init__(self):
        self.config = {}
        self.queue = None


    def read_config(self, filename):
        """Read from a config YAML file and set up queues."""
        with io.open(filename, 'r') as f:
            self.config = yaml.safe_load(f.read())

        if self.config.get('module').lower() == 'sqs':
            self.queue = SQSInterface(
                self.config['aws_access_key_id'],
                self.config['aws_secret_access_key'],
                self.config['aws_region'],
                in_queue=self.config.get('in_queue'),
                out_queue=self.config.get('out_queue'))
        elif self.config.get('module').lower() == 'beanstalk':
            self.queue = BeanstalkInterface(
                self.config['host'],
                self.config['port'],
                in_queue=self.config.get('in_queue'),
                out_queue=self.config.get('out_queue'))
        else:
            raise threatingestor.exceptions.IngestorError('Invalid self.config.')


    def run_forever(self):
        """Do work forever.

        Note: There is no sleep here! If anything fails, it might spin out of
        control.
        """
        while True:
            content = self.do_work(self.queue.read_one())
            self.queue.write_one(content)


    def do_work(self, job):
        """Implement in child class.

        One or zero jobs are passed in from the queue.
        One or zero jobs are returned and sent to the queue.
        """
        raise NotImplementedError()


class SQSInterface:
    """A consistent Queue interface for SQS."""
    def __init__(self, aws_access_key_id, aws_secret_access_key,
                 aws_region, in_queue=None, out_queue=None):
        """Set up SQS connections.

        :param aws_access_key_id: AWS access key ID.
        :param aws_secret_access_key: AWS secret access key.
        :param aws_region: AWS region string.
        :param in_queue: Optional input queue name.
        :param out_queue: Optional output queue name.
        """
        self.in_queue = None
        self.out_queue = None

        if in_queue:
            resource = boto3.resource('sqs', region_name=aws_region,
                                      aws_access_key_id=aws_access_key_id,
                                      aws_secret_access_key=aws_secret_access_key)
            self.in_queue = resource.get_queue_by_name(QueueName=in_queue)

        if out_queue:
            client = boto3.client('sqs', region_name=aws_region,
                                  aws_access_key_id=aws_access_key_id,
                                  aws_secret_access_key=aws_secret_access_key)
            self.out_queue = client.get_queue_url(QueueName=out_queue)['QueueUrl']


    def read_one(self):
        """Read one or zero messages from the queue.

        :returns: Message body or None.
        """
        if not self.in_queue:
            return

        messages = self.in_queue.receive_messages(
            MaxNumberOfMessages=1,
            WaitTimeSeconds=20)
        if not messages:
            return

        job = json.loads(messages[0].body)
        messages[0].delete()
        return job


    def write_one(self, content):
        """Write one message to the queue, if it exists."""
        if not self.out_queue or not content:
            return

        return self.out_queue.send_message(
            QueueUrl=self.out_queue,
            DelaySeconds=0,
            MessageBody=json.dumps(content)
        )


class BeanstalkInterface:
    """A consistent Queue interface for Beanstalk."""
    def __init__(self, host, port, in_queue=None, out_queue=None):
        """Set up Beanstalk connections.

        :param host: Beanstalk host.
        :param port: Beanstalk port.
        :param in_queue: Optional input queue name.
        :param out_queue: Optional output queue name.
        """
        self.in_queue = None
        self.out_queue = None

        if in_queue:
            self.in_queue = greenstalk.Client(host, port, watch=in_queue)

        if out_queue:
            self.out_queue = greenstalk.Client(host, port, use=out_queue)


    def read_one(self):
        """Read one or zero messages from the queue.

        :returns: Message body or None.
        """
        if not self.in_queue:
            return

        message = self.in_queue.reserve()
        job = json.loads(message.body)
        self.in_queue.delete(message)
        return job


    def write_one(self, content):
        """Write one message to the queue, if it exists."""
        if not self.out_queue or not content:
            return

        self.out_queue.put(json.dumps(content))



import json

import threatingestor.artifacts
from threatingestor.operators import abstract_json
from threatingestor.exceptions import DependencyError

try:
    import greenstalk
except ImportError:
    raise DependencyError("Dependency greenstalk required for Beanstalk operator is not installed")

class Plugin(abstract_json.AbstractPlugin):
    """Operator for Beanstalk work queue."""
    def __init__(self, host, port, queue_name, artifact_types=None,
            filter_string=None, allowed_sources=None, **kwargs):
        """Beanstalk operator."""
        self.queue = greenstalk.Client(host, port, use=queue_name)

        super(Plugin, self).__init__(artifact_types=artifact_types,
                                     filter_string=filter_string,
                                     allowed_sources=allowed_sources,
                                     **kwargs)
        self.artifact_types = artifact_types or [
            threatingestor.artifacts.URL,
        ]


    def _put(self, content):
        """Send content to a Beanstalk queue."""
        return self.queue.put(
            json.dumps(content)
        )



import requests
import datetime
import regex as re
from bs4 import BeautifulSoup
from urllib.parse import urlparse

from threatingestor.sources import Source

class Plugin(Source):

    def __init__(self, name, url, filter=None, path=None):
        self.name = name
        self.url = url
        self.filter = filter
        self.path = path

    def run(self, saved_state):
        # Configures sitemap parsing
        response = requests.get(self.url)
        xml = BeautifulSoup(response.text, "lxml-xml")

        urls = xml.find_all("url")
        text = ""
        artifacts = []

        for u in urls:
            if saved_state and xml.find("lastmod") and saved_state <= str(u.findNext("lastmod").text).split("T", 1)[0]:
                continue

            # Extracts only the 'loc' tag from the xml
            if xml.find("loc"):
                loc = u.findNext("loc").text
                response = requests.get(loc)
                soup = BeautifulSoup(response.text, 'html.parser')
            else:
                continue

            if response.status_code >= 400:
                continue

            [x.unwrap() for x in soup.find_all('strong')]
            [x.unwrap() for x in soup.find_all('b')]
            [x.unwrap() for x in soup.find_all('em')]
            [x.unwrap() for x in soup.find_all('i')]
            soup = BeautifulSoup(soup.decode(), 'html.parser')

            if self.filter is not None:
                # Regex input via config.yml
                # Example: security|threat|malware
                xml_query = re.compile(r"{0}".format(self.filter)).findall(str(self.filter.split('|')))

                # Iterates over the regex output to locate all provided keywords
                for x in xml_query:
                    # Uses a path instead of a keyword
                    if self.path is not None:
                        if self.path in loc:
                            text = soup.get_text(separator=' ').split('Indicators of Compromise')[-1]
                            artifacts += self.process_element(content=text, reference_link=str(loc), include_nonobfuscated=True)

                    # Only filters using a keyword
                    if self.path is None:
                        if x in loc:
                            text = soup.get_text(separator=' ').split('Indicators of Compromise')[-1]
                            artifacts += self.process_element(content=text, reference_link=str(loc), include_nonobfuscated=True)

            elif self.filter is None and self.path is not None:
                # Filters only by path in XML loc, no set filter
                # Default: /path/name/*

                if self.path in loc:
                    text = soup.get_text(separator=' ').split('Indicators of Compromise')[-1]
                    artifacts += self.process_element(content=text, reference_link=str(loc), include_nonobfuscated=True)
            
            else:
                # Locates all blog links within the sitemap
                if "blog" in loc:
                    text = soup.get_text(separator=' ').split('Indicators of Compromise')[-1]
                    artifacts += self.process_element(content=text, reference_link=str(loc), include_nonobfuscated=True)

            if xml.find("lastmod"):
                saved_state = str(u.findNext("lastmod").text).split("T", 1)[0]
            else:
                saved_state = str(datetime.date.today())

        return saved_state, artifacts



import threatingestor.artifacts
from threatingestor.operators import Operator
from threatingestor.exceptions import DependencyError

try:
    import pymysql
except ImportError:
    raise DependencyError("Dependency pymysql required for MySQL operator is not installed")

class Plugin(Operator):
    """Operator for MySQL."""
    def __init__(self, host, database, table, user=None, password='', port=3306,
                 artifact_types=None, filter_string=None, allowed_sources=None):
        """MySQL operator."""
        super(Plugin, self).__init__(artifact_types, filter_string, allowed_sources)
        self.artifact_types = artifact_types or [
            threatingestor.artifacts.Domain,
            threatingestor.artifacts.Hash,
            threatingestor.artifacts.IPAddress,
            threatingestor.artifacts.URL,
            threatingestor.artifacts.YARASignature,
            threatingestor.artifacts.Task,
        ]

        # Connect to SQL and set up the tables if they aren't already.
        self.table = table
        self.host = host
        self.database = database
        self.user = user
        self.password = password
        self.port = port

        self._create_table()


    def _create_table(self):
        """Create the table defined in the config if it does not already exist."""
        query = f"""
            CREATE TABLE IF NOT EXISTS `{self.table}` (
                `id` INT UNSIGNED NOT NULL AUTO_INCREMENT PRIMARY KEY,
                `artifact` TEXT,
                `artifact_type` VARCHAR(50),
                `reference_link` TExT,
                `reference_text` TEXT,
                `created_date` DATETIME DEFAULT CURRENT_TIMESTAMP,
                `state` TEXT
            ) 
        """
        # See note in process() on why we connect to SQL here.
        self.sql = pymysql.connect(host=self.host, port=self.port, user=self.user,
                                   password=self.password, database=self.database)
        self.cursor = self.sql.cursor()
        self.cursor.execute(query)
        self.sql.commit()
        self.cursor.close()


    def _insert_artifact(self, artifact):
        """Insert the given artifact into the table."""
        type_name = artifact.__class__.__name__.lower()
        query = f"""
            INSERT IGNORE INTO `{self.table}` (
                `artifact`,
                `artifact_type`,
                `reference_link`,
                `reference_text`,
                `state`
            )
            VALUES (%s, %s, %s, %s, NULL)
        """
        self.cursor.execute(query, (
            str(artifact),
            type_name,
            artifact.reference_link,
            artifact.reference_text
        ))
        self.sql.commit()


    def process(self, artifacts):
        """Override parent method to better handle SQL cursor."""
        # Connect to MySQL and call the parent method.
        # Note: We do this here and in _create_table rather than in the constructor
        # because it seems the SQL connection is prone to timeouts, and
        # establishing the connection here is the most efficient way to avoid that.
        self.sql = pymysql.connect(host=self.host, port=self.port, user=self.user,
                                   password=self.password, database=self.database)
        self.cursor = self.sql.cursor()
        super(Plugin, self).process(artifacts)

        # Close the connection
        self.cursor.close()


    def handle_artifact(self, artifact):
        """Operate on a single artifact."""
        self._insert_artifact(artifact)



import unittest
from unittest.mock import Mock, patch
import json

import threatingestor.operators.beanstalk
import threatingestor.artifacts


class TestSQS(unittest.TestCase):

    @patch('greenstalk.Client')
    def setUp(self, boto3_client):
        self.beanstalk = threatingestor.operators.beanstalk.Plugin('a', 'b', 'c', test='{domain}')

    def test_put_sends_string(self):
        self.beanstalk.handle_artifact(threatingestor.artifacts.URL('http://example.com', 'a'))
        self.beanstalk.queue.put.assert_called_once_with('{"test": "example.com"}')



from __future__ import absolute_import

import re
import twitter
from loguru import logger

from threatingestor.operators import Operator
import threatingestor.artifacts
import threatingestor.exceptions

TWEET_URL = re.compile(r'https://twitter\.com/\w{1,15}/status/\d+')

class Plugin(Operator):
    """Operator for Twitter."""
    def __init__(self, api_key, api_secret_key, access_token, access_token_secret, status, **kwargs):
        self.api = twitter.Twitter(auth=twitter.OAuth(access_token, access_token_secret, api_key, api_secret_key))
        self.status = status

        # Validate status, for better error handling.
        if not isinstance(self.status, str):
            raise threatingestor.exceptions.IngestorError(f"Invalid 'status' config: {self.status}")

        super(Plugin, self).__init__(kwargs.get('artifact_types'),
                                     kwargs.get('filter_string'),
                                     kwargs.get('allowed_sources'))
        self.artifact_types = kwargs.get('artifact_types') or [
            threatingestor.artifacts.URL,
            threatingestor.artifacts.Domain,
            threatingestor.artifacts.Hash,
            threatingestor.artifacts.IPAddress,
        ]


    def handle_artifact(self, artifact):
        """Operate on a single artifact."""
        status = artifact.format_message(self.status)

        # If artifact.reference_link is a tweet permalink, quote-tweet it.
        quote_tweet = None
        if TWEET_URL.match(artifact.reference_link):
            quote_tweet = artifact.reference_link

        self._tweet(status, quote_tweet=quote_tweet)


    def _tweet(self, status, quote_tweet=None):
        """Send content to Twitter as a status update."""
        try:
            return self.api.statuses.update(status=status,
                                            attachment_url=quote_tweet,
                                            tweet_mode='extended')
        except twitter.api.TwitterHTTPError as e:
            logger.warning(f"Twitter API Error: {e}")
            return None



import sqlite3
import hug

DATABASE = 'artifacts.db'


INDEX_HTML = """
<html>
<head>
    <script
      src="https://code.jquery.com/jquery-3.3.1.min.js"
      integrity="sha256-FgpCb/KJQlLNfOu91ta32o/NMZxltwRo8QtmkMRdAu8="
      crossorigin="anonymous"></script>

    <script type="text/javascript">
        $(document).ready(function() {
            $.ajax('/api/index').done(function(tables) {
                tables.forEach(function(table) {
                    $("body").append(`<p><a href="/${table}">${table}</a></p>`);
                });
            });
        });
    </script>
</head>
<body>
<h1>Artifact types</h1>
</body>
</html>
"""


LIST_HTML = """
<html>
<head>
    <script
      src="https://code.jquery.com/jquery-3.3.1.min.js"
      integrity="sha256-FgpCb/KJQlLNfOu91ta32o/NMZxltwRo8QtmkMRdAu8="
      crossorigin="anonymous"></script>
      <link rel="stylesheet" type="text/css" href="https://cdn.datatables.net/v/dt/dt-1.10.18/datatables.min.css"/>

    <script type="text/javascript" src="https://cdn.datatables.net/v/dt/dt-1.10.18/datatables.min.js"></script>
    <script type="text/javascript">
        $(document).ready(function() {
            $('#table').DataTable({
                "ajax": {
                    "url": `/api${window.location.pathname}`,
                    "dataSrc": "",
                },
                "columns": [
                    {"data": "artifact"},
                    {"data": "reference_link"},
                    {"data": "reference_text"},
                    {"data": "created_date"},
                    {"data": "state"}
                ]
            });
        });
    </script>

</head>
<body>
    <table id='table'>
        <thead>
            <tr>
                <th>Artifact</th>
                <th>Reference Link</th>
                <th>Reference Text</th>
                <th>Created Date</th>
                <th>State</th>
            </tr>
        </thead>
        <tbody></tbody>
    </table>
</body>
</html>
"""


@hug.get('/api/{table}', output=hug.output_format.json)
def list_view(table):
    cursor.execute('SELECT name FROM sqlite_master WHERE type="table"')
    tables = [e[0] for e in cursor.fetchall()]

    if table == 'index':
        return tables

    elif table and table in tables:
        cursor.execute(f'SELECT * FROM {table}')

        data = []
        columns = [c[0] for c in cursor.description]
        for row in cursor.fetchall():
            data.append({k: v for k, v in zip(columns, row)})

        return data

    elif table:
        return f"No table {table}"

    else:
        return tables


@hug.get('/{table}', output=hug.output_format.html)
def html_view(table):
    if table:
        return LIST_HTML
    else:
        return INDEX_HTML


db = sqlite3.connect(DATABASE)
cursor = db.cursor()


if __name__ == "__main__":
    print("usage: hug -m threatingestor.extras.webapp")



import sys
import io
import unittest
from unittest.mock import mock_open, patch

import threatingestor.config
import threatingestor.artifacts
import threatingestor.sources.rss
import threatingestor.operators.csv
import threatingestor.operators.threatkb


class TestConfig(unittest.TestCase):

    def setUp(self):
        m = mock_open()
        with patch('threatingestor.config.io.open', m, create=True):
            self.config = threatingestor.config.Config('test')

    def test_daemon_returns_main_daemon_bool(self):
        # Mock true/false values for daemon variable.
        self.config.config = {
            'general': {
                'daemon': True,
            }
        }
        self.assertTrue(self.config.daemon())

        self.config.config = {
            'general': {
                'daemon': False,
            }
        }
        self.assertFalse(self.config.daemon())

    def test_state_path_returns(self):
        self.config.config = {
            'general': {
                'state_path': "/home/user/test",
            }
        }

        self.assertEqual(self.config.state_path(), "/home/user/test")

    def test_logging_returns_full_section(self):
        self.config.config = {
            'logging': {
                'arbitrary': 'test',
            }
        }

        self.assertEqual(self.config.logging(), {'arbitrary': 'test'})

    def test_notifiers_returns_full_section(self):
        self.config.config = {
            'notifiers': {
                'arbitrary': 'test',
            }
        }

        self.assertEqual(self.config.notifiers(), {'arbitrary': 'test'})

    def test_statsd_returns_full_section(self):
        self.config.config = {
            'statsd': {
                'arbitrary': 'test',
            }
        }

        self.assertEqual(self.config.statsd(), {'arbitrary': 'test'})

    def test_sleep_returns_main_sleep_int(self):
        self.config.config = {
            'general': {
                'sleep': 10,
            }
        }
        self.assertEqual(self.config.sleep(), 10)

        self.config.config = {
            'general': {
                'sleep': 90,
            }
        }
        self.assertEqual(self.config.sleep(), 90)

    def test_credentials_returns_dictionary_of_credentials(self):
        self.config.config = {
            'credentials': [
                {
                    'name': 'twitter-myuser',
                    'token': 'EXAMPLE',
                    'token_key': 'EXAMPLE',
                    'con_secret_key': 'EXAMPLE',
                    'con_secret': 'EXAMPLE'
                },
            ]
        }

        expected = {
            'name': 'twitter-myuser',
            'token': 'EXAMPLE',
            'token_key': 'EXAMPLE',
            'con_secret_key': 'EXAMPLE',
            'con_secret': 'EXAMPLE'
        }

        self.assertEqual(self.config.credentials('twitter-myuser') , expected)

    def test_sources_returns_list_of_all_source_tuples(self):
        self.config.config = {
            'sources': [
                {
                    'name': 'test-one',
                    'module': 'rss',
                    'url': 'example',
                },
                {
                    'name': 'test-two',
                    'module': 'rss',
                    'url': 'example',
                },
            ]
        }
        expected_sources = [
            ('test-one', threatingestor.sources.rss.Plugin, {'name': 'test-one', 'url': 'example'}),
            ('test-two', threatingestor.sources.rss.Plugin, {'name': 'test-two', 'url': 'example'}),
        ]
        self.assertEqual(self.config.sources(), expected_sources)

    def test_sources_list_includes_credentials(self):
        self.config.config = {
            'credentials': [
                {
                    'name': 'mycreds',
                    'token': 'mytoken',
                },
            ],
            'sources': [
                {
                    'name': 'test-one',
                    'module': 'rss',
                    'url': 'example',
                    'credentials': 'mycreds',
                },
                {
                    'name': 'test-two',
                    'module': 'rss',
                    'url': 'example',
                },
            ]
        }
        expected_sources = [
            ('test-one', threatingestor.sources.rss.Plugin,
                {'name': 'test-one', 'url': 'example', 'token': 'mytoken'}),
            ('test-two', threatingestor.sources.rss.Plugin, {'name': 'test-two', 'url': 'example'}),
        ]
        self.assertEqual(self.config.sources(), expected_sources)

    def test_sources_excludes_internal_options_from_kwargs(self):
        self.config.config = {
            'sources': [
                {
                    'name': 'test-one',
                    'module': 'rss',
                    'url': 'example',
                    'saved_state': 'test',
                },
            ]
        }

        expected_sources = [
            ('test-one', threatingestor.sources.rss.Plugin, {'url': 'example','name': 'test-one'}),
        ]
        self.assertEqual(self.config.sources(), expected_sources)

    def test_operators_returns_list_of_all_operator_tuples(self):
        self.config.config = {
            'operators': [
                {
                    'name': 'test-one',
                    'module': 'csv',
                    'filename': 'output.csv',
                },
                {
                    'name': 'test-three',
                    'module': 'csv',
                    'filename': 'output.csv',
                },
            ]
        }

        expected_operators = [
            ('test-one', threatingestor.operators.csv.Plugin, {'filename': 'output.csv'}),
            ('test-three', threatingestor.operators.csv.Plugin, {'filename': 'output.csv'}),
        ]
        self.assertEqual(self.config.operators(), expected_operators)

    def test_operators_list_includes_credentials(self):
        self.config.config = {
            'credentials': [
                {
                    'name': 'mycreds',
                    'token': 'mytoken',
                },
            ],
            'operators': [
                {
                    'name': 'test-one',
                    'module': 'csv',
                    'filename': 'output.csv',
                    'credentials': 'mycreds'
                },
                {
                    'name': 'test-three',
                    'module': 'csv',
                    'filename': 'output.csv',
                },
            ],
        }

        expected_operators = [
            ('test-one', threatingestor.operators.csv.Plugin, {'filename': 'output.csv', 'token': 'mytoken'}),
            ('test-three', threatingestor.operators.csv.Plugin, {'filename': 'output.csv'}),
        ]

        self.assertEqual(self.config.operators(), expected_operators)

    def test_operators_excludes_internal_options_from_kwargs(self):
        self.config.config = {
            'operators': [
                {
                    'name': 'test-one',
                    'module': 'csv',
                    'filename': 'output.csv',
                    'allowed_sources': ["mysource", "myothersource"],
                    'filter': '([^\.]google.com$|google.com[^/])',
                    'artifact_types': ["URL", "Domain"],
               },
            ]

        }

        expected_operators = [
            ('test-one', threatingestor.operators.csv.Plugin, {
                'filename': 'output.csv',
                'allowed_sources': ["mysource", "myothersource"],
                'filter_string': '([^\.]google.com$|google.com[^/])',
                'artifact_types': [
                    threatingestor.artifacts.URL,
                    threatingestor.artifacts.Domain,
                ],
            }),
        ]

        self.assertEqual(self.config.operators(), expected_operators)

    def test_load_plugin_returns_plugin_class(self):
        self.assertEqual(self.config._load_plugin(threatingestor.config.SOURCE, 'rss'),
                         threatingestor.sources.rss.Plugin)
        self.assertEqual(self.config._load_plugin(threatingestor.config.OPERATOR, 'csv'),
                         threatingestor.operators.csv.Plugin)

    @patch('importlib.import_module')
    def test_load_plugin_raises_pluginerror_if_no_plugin(self, import_module):
        # module doesn't exist
        import_module.side_effect = ImportError

        with self.assertRaises(threatingestor.exceptions.PluginError):
            self.config._load_plugin(threatingestor.config.OPERATOR, 'csv')

        # Plugin class doesn't exist
        import_module.side_effect = AttributeError

        with self.assertRaises(threatingestor.exceptions.PluginError):
            self.config._load_plugin(threatingestor.config.OPERATOR, 'csv')



import unittest
from unittest.mock import patch, ANY as MOCK_ANY

import threatingestor.operators.misp
import threatingestor.artifacts


class TestThreatKB(unittest.TestCase):

    @patch('pymisp.ExpandedPyMISP')
    def setUp(self, ExpandedPyMISP):
        self.misp = threatingestor.operators.misp.Plugin('a', 'b')

    @patch('pymisp.ExpandedPyMISP')
    def test_tags_are_set_if_passed_in_else_default(self, ExpandedPyMISP):
        self.assertEqual(self.misp.tags, ['type:OSINT'])
        self.assertEqual(
            threatingestor.operators.misp.Plugin('a', 'b', tags=['c', 'd']).tags,
            ['c', 'd'],
        )

    def test_create_event_creates_event_and_objects(self):
        event = self.misp._create_event(
            threatingestor.artifacts.Domain(
                'test.com',
                'name',
                reference_link='link',
                reference_text='text',
            )
        )
        self.misp._update_or_create_event(event)
        self.misp.api.add_event.assert_called_once_with(event)

    def test_handle_domain_creates_domain(self):
        domain = threatingestor.artifacts.Domain('test.com', '', '')

        event = self.misp._create_event(domain)
        event = self.misp.handle_domain(domain, event)
        self.assertEqual(event.Attribute[0].value, str(domain))

    def test_handle_hash_creates_hash(self):
        hash = threatingestor.artifacts.Hash(
            '68b329da9893e34099c7d8ad5cb9c940', '', '')
        event = self.misp._create_event(hash)
        event = self.misp.handle_hash(hash, event)
        self.assertEqual(event.Attribute[0].value, str(hash))

        hash = threatingestor.artifacts.Hash(
            'adc83b19e793491b1c6ea0fd8b46cd9f32e592fc', '', '')
        event = self.misp._create_event(hash)
        event = self.misp.handle_hash(hash, event)
        self.assertEqual(event.Attribute[0].value, str(hash))

        hash = threatingestor.artifacts.Hash(
            '01ba4719c80b6fe911b091a7c05124b64eeece964e09c058ef8f9805daca546b', '', '')
        event = self.misp._create_event(hash)
        event = self.misp.handle_hash(hash, event)
        self.assertEqual(event.Attribute[0].value, str(hash))

        hash = threatingestor.artifacts.Hash('invalid', '', '')
        event = self.misp._create_event(hash)
        event = self.misp.handle_hash(hash, event)
        self.assertEqual(event.Attribute, [])

    def test_handle_ipaddress_creates_ipaddress(self):
        ipaddress = threatingestor.artifacts.IPAddress('123.123.123.123', '', '')
        event = self.misp._create_event(ipaddress)
        event = self.misp.handle_ipaddress(ipaddress, event)
        self.assertEqual(event.Attribute[0].value, str(ipaddress))

    def test_handle_url_creates_url(self):
        url = threatingestor.artifacts.URL('http://example.com', '', '')
        event = self.misp._create_event(url)
        event = self.misp.handle_url(url, event)
        self.assertEqual(event.Attribute[0].value, str(url))

    def test_handle_yarasignature_creates_yarasignature(self):
        yara = threatingestor.artifacts.YARASignature('test', '', '')
        event = self.misp._create_event(yara)
        event = self.misp.handle_yarasignature(yara, event)
        self.assertEqual(event.Attribute[0].value, str(yara))

    def test_handle_artifact_creates_event(self):
        artifact = threatingestor.artifacts.URL('http://example.com', '', '')
        event = self.misp.handle_artifact(artifact)
        self.misp.api.add_event.assert_called_once()

    @patch('pymisp.ExpandedPyMISP')
    def test_artifact_types_are_set_if_passed_in_else_default(self, ExpandedPyMISP):
        artifact_types = [
            threatingestor.artifacts.IPAddress,
            threatingestor.artifacts.URL,
        ]
        self.assertEqual(
            threatingestor.operators.misp.Plugin('a', 'b', artifact_types=artifact_types).artifact_types,
            artifact_types,
        )
        self.assertEqual(threatingestor.operators.misp.Plugin('a', 'b').artifact_types, [
            threatingestor.artifacts.Domain,
            threatingestor.artifacts.Hash,
            threatingestor.artifacts.IPAddress,
            threatingestor.artifacts.URL,
            threatingestor.artifacts.YARASignature
        ])

    @patch('pymisp.ExpandedPyMISP')
    def test_filter_string_and_allowed_sources_are_set_if_passed_in(self, ExpandedPyMISP):
        self.assertEqual(
            threatingestor.operators.misp.Plugin('a', 'b', filter_string='test').filter_string,
            'test',
        )
        self.assertEqual(
            threatingestor.operators.misp.Plugin('a', 'b', allowed_sources=['test-one']).allowed_sources,
            ['test-one'],
        )



import unittest
from unittest.mock import Mock, patch
import json


import threatingestor.sources.abstract_json
import threatingestor.artifacts
import threatingestor.exceptions


class TestJSON(unittest.TestCase):

    def setUp(self):
        self.abs_json = threatingestor.sources.abstract_json.AbstractPlugin("name", [], "")

    def test_run(self):
        content_list = [{"testKey": "testValue"}]
        self.abs_json.get_objects = lambda x: (None, content_list)

        saved_state, artifact_list = self.abs_json.run(None)

        self.assertEqual(saved_state, None)

    def test_run_uses_reference_path_if_possible(self):
        content_list = [{"testContent": "http://example.com", "testRef": "myReference"}]
        abs_json = threatingestor.sources.abstract_json.AbstractPlugin("name", ["testContent"], "testRef")
        abs_json.get_objects = lambda x: (None, content_list)

        saved_state, artifact_list = abs_json.run(None)

        self.assertEqual(3, len(artifact_list))
        for artifact in artifact_list:
            self.assertEqual('myReference', artifact.reference_link)

    def test_run_uses_name_if_reference_path_not_found(self):
        content_list = [{"testContent": "http://example.com", "testRef": "myReference"}]
        abs_json = threatingestor.sources.abstract_json.AbstractPlugin("name", ["testContent"], "badRef")
        abs_json.get_objects = lambda x: (None, content_list)

        saved_state, artifact_list = abs_json.run(None)

        self.assertEqual(3, len(artifact_list))
        for artifact in artifact_list:
            self.assertEqual('name', artifact.reference_link)

    def test_run_extracts_artifacts(self):
        content_list = [{"testContent": "http://example.com", "testRef": "myReference"}]
        abs_json = threatingestor.sources.abstract_json.AbstractPlugin("name", ["testContent"], "testRef")
        abs_json.get_objects = lambda x: (None, content_list)

        saved_state, artifact_list = abs_json.run(None)

        self.assertEqual(3, len(artifact_list))
        self.assertIn('http://example.com', [str(x) for x in artifact_list])



import unittest
from unittest.mock import patch, ANY as MOCK_ANY

import threatingestor.operators.threatkb
import threatingestor.artifacts


class TestThreatKB(unittest.TestCase):

    @patch('threatkb.ThreatKB')
    def setUp(self, ThreatKB):
        self.threatkb = threatingestor.operators.threatkb.Plugin('a', 'b', 'c', 'd')

    def test_handle_domain_creates_domain(self):
        self.threatkb.handle_artifact(threatingestor.artifacts.Domain('test.com', '', ''))
        self.threatkb.api.create.assert_called_once_with('c2dns', MOCK_ANY)

    def test_handle_ipaddress_creates_ipaddress(self):
        self.threatkb.handle_artifact(threatingestor.artifacts.IPAddress('123.123.123.123', '', ''))
        self.threatkb.api.create.assert_called_once_with('c2ips', MOCK_ANY)

    def test_handle_yarasignature_creates_yarasignature(self):
        self.threatkb.handle_artifact(threatingestor.artifacts.YARASignature('test', '', ''))
        self.threatkb.api.create.assert_called_once_with('import', MOCK_ANY)

    def test_artifact_types_are_set_if_passed_in_else_default(self):
        artifact_types = [threatingestor.artifacts.IPAddress, threatingestor.artifacts.URL]
        self.assertEqual(threatingestor.operators.threatkb.Plugin('a', 'b', 'c', 'd', artifact_types=artifact_types).artifact_types, artifact_types)
        self.assertEqual(threatingestor.operators.threatkb.Plugin('a', 'b', 'c', 'd').artifact_types, [threatingestor.artifacts.Domain, threatingestor.artifacts.IPAddress,
                threatingestor.artifacts.YARASignature])

    def test_filter_string_and_allowed_sources_are_set_if_passed_in(self):
        self.assertEqual(threatingestor.operators.threatkb.Plugin('a', 'b', 'c', 'd', filter_string='test').filter_string, 'test')
        self.assertEqual(threatingestor.operators.threatkb.Plugin('a', 'b', 'c', 'd', allowed_sources=['test-one']).allowed_sources, ['test-one'])

    def test_handle_task_creates_task(self):
        self.threatkb.handle_artifact(threatingestor.artifacts.Task('', '', ''))
        self.threatkb.api.create.assert_called_once_with('tasks', MOCK_ANY)




import re
from loguru import logger


class Operator:
    """Base class for all Operator plugins.

    Note: This is an abstract class. You must extend ``__init__`` and call
    ``super`` to ensure this class's constructor is called. You must override
    ``handle_artifact`` with the same signature. You may define additional
    ``handle_{artifact_type}`` methods as needed (see the threatkb operator for
    an example) - these methods are purely convention, and are not required.

    When adding additional methods to child classes, consider prefixing the
    method name with an underscore to denote a ``_private_method``. Do not
    override other existing methods from this class.
    """
    def __init__(self, artifact_types=None, filter_string=None, allowed_sources=None):
        """Override this constructor in child classes.

        The arguments above (artifact_types, filter_string, allowed_sources)
        should be accepted explicity as above, in all child classes.

        Additional arguments should be added: url, auth, etc, whatever is
        needed to set up the object.

        Each operator should default self.artifact_types to a list of Artifacts
        supported by the plugin, and allow passing in artifact_types to
        overwrite that default.

        Example:

        >>> self.artifact_types = artifact_types or [
        ...     artifacts.IPAddress,
        ...     artifacts.Domain,
        ... ]

        It's recommended to call this __init__ method via super from all child
        classes. Remember to do so *before* setting any default artifact_types.
        """
        self.artifact_types = artifact_types or []
        self.filter_string = filter_string or ''
        self.allowed_sources = allowed_sources or []


    def handle_artifact(self, artifact):
        """Override with the same signature.

        :param artifact: A single ``Artifact`` object.
        :returns: None (always ignored)
        """
        raise NotImplementedError()


    def _artifact_is_allowed(self, artifact):
        """Returns True iff this artifact is allowed by this plugin's filters."""
        # Must be in allowed_types.
        if not any(isinstance(artifact, t) for t in self.artifact_types):
            return False

        # Must match the filter string.
        if not artifact.match(self.filter_string):
            return False

        # Must be in allowed_sources, if set.
        if self.allowed_sources and not any(
                [re.compile(p).search(artifact.source_name)
                 for p in self.allowed_sources]):
            return False

        return True


    def process(self, artifacts):
        """Process all applicable artifacts."""
        for artifact in artifacts:
            if self._artifact_is_allowed(artifact):
                self.handle_artifact(artifact)



import jsonpath_rw

from threatingestor.sources import Source

class AbstractPlugin(Source):

    def __init__(self, name, paths, reference=None, **kwargs):
        """Set up JSON path expressions.

        Extend in child class.
        """
        self.name = name
        # Parse paths into path expressions.
        self.path_expressions = [jsonpath_rw.parse(path) for path in paths]
        # Try to parse reference as a path, fall back to name.
        self.reference = jsonpath_rw.parse(reference) if reference else None


    def get_objects(self, saved_state):
        """Produce an iterable of dict or list objectcs containing raw content to process.

        Override in child class."""
        raise NotImplementedError()


    def run(self, saved_state):
        """Run and return (saved_state, list(Artifact))"""
        artifact_list = []

        saved_state, content_list = self.get_objects(saved_state)
        for content in content_list:
            # Iterate over each piece of "content", extracting from every path of interest.
            for path_expression in self.path_expressions:
                matches = path_expression.find(content)

                # Try to get a single reference, fall back to name again.
                try:
                    reference = self.reference.find(content)[0].value
                except (IndexError, AttributeError):
                    reference = self.name

                # Extract artifacts.
                for match in matches:
                    artifact_list += self.process_element(match.value, reference, include_nonobfuscated=True)

        return saved_state, artifact_list



import unittest

from threatingestor.state import State


class TestState(unittest.TestCase):

    def setUp(self):
        self.state= State(":memory:")

    def test_state_updates_known_state(self):
        result = None
        initial= ["STATE","NAME"]
        expected = ["NEWSTATE","NAME"]

        #inital save state
        self.state.save_state(initial[1],initial[0])
        beforeUpdate = self.state.get_state(expected[1])

        #rewrite over same state
        self.state.save_state( expected[1], expected[0])
        afterUpdate= self.state.get_state(expected[1])

        self.assertEqual(afterUpdate, expected[0])

    def test_state_not_exists (self):
        result = None
        initial= ["STATE","NAME"]
        expected= None

        result= self.state.get_state(initial[1])

        self.assertEqual(result, expected)

    def test_state_saves(self):
        result = None
        expected = ["STATE","NAME"]
        self.state.save_state(expected[1],expected[0])

        result = self.state.get_state(expected[1])

        self.assertEqual(result, expected[0])



import unittest
from pathlib import Path

from threatingestor.whitelist import Whitelist


class TestState(unittest.TestCase):
    def setUp(self):
        parent = Path(__file__).parent.absolute()
        path = parent / "fixtures/whitelist.json"
        self.whitelist = Whitelist(paths=[str(path)])

    def test_contains(self):
        self.assertTrue(self.whitelist.contains("00-tv.com"))
        self.assertFalse(self.whitelist.contains("example.com"))



import unittest
from unittest.mock import Mock, patch

import threatingestor.sources.beanstalk


class TestBeanstalk(unittest.TestCase):

    @patch('greenstalk.Client')
    def setUp(self, greenstalk_client):
        self.beanstalk = threatingestor.sources.beanstalk.Plugin('a', 'b', 'c', 'd', ['content'])

    def test_run_reads_messages_deletes_returns_artifacts(self):
        message = Mock()
        message.body = '{"content": "http://example.mock/path"}'
        self.beanstalk.queue.reserve.return_value = message

        saved_state, artifact_list = self.beanstalk.run(None)

        self.assertEqual(saved_state, None)
        self.beanstalk.queue.reserve.assert_called_with(timeout=1)
        self.beanstalk.queue.delete.assert_called()
        self.assertIn('http://example.mock/path', [str(x) for x in artifact_list])



import unittest
from unittest.mock import Mock, patch

import threatingestor

class TestIngestor(unittest.TestCase):

    @patch('threatingestor.config.Config')
    def setUp(self, Config):
        mock_source_operator = Mock()
        mock_source_operator.return_value.run.return_value = (1,[])

        Config.return_value.sources.return_value = [
            ['test-twitter', mock_source_operator, {'q': 'test'}],
            ['test-rss', mock_source_operator, {'url': 'test'}],
        ]
        Config.return_value.operators.return_value = [
            ['test-threatkb', mock_source_operator, {'url': 'test'}],
            ['test-csv', mock_source_operator, {'filename': 'test'}],
        ]

        Config.return_value.state_path.return_value = ":memory:"
        self.app = threatingestor.Ingestor('test')
        self.app.statedb = Mock()

    @patch('threatingestor.config.Config')
    def test_init_creates_sources_operators_dicts(self, Config):
        attrs = {
            'sources.return_value': [
                ['test-twitter', Mock, {'q': 'test'}],
                ['test-rss', Mock, {'url': 'test'}],
            ],
            'operators.return_value': [
                ['test-threatkb', Mock, {'url': 'test'}],
                ['test-csv', Mock, {'filename': 'test'}],
            ],
        }
        Config.return_value.configure_mock(**attrs)

        Config.return_value.state_path.return_value = ":memory:"

        app = threatingestor.Ingestor('test')
        self.assertEqual(app.sources['test-twitter'].q, 'test')
        self.assertEqual(app.operators['test-csv'].filename, 'test')
        self.assertEqual(len(app.sources), 2)
        self.assertEqual(len(app.operators), 2)

    def test_run_checks_config_daemon(self):
        self.app.config.daemon.return_value = False
        self.app.run()
        self.app.config.daemon.assert_called_once()

    def test_run_once_calls_run_process_save_state(self):
        self.app.sources['test-twitter'].process.assert_not_called()
        self.app.sources['test-twitter'].run.assert_not_called()
        self.app.statedb.save_state.assert_not_called()

        self.app.run_once()
        self.app.sources['test-twitter'].process.assert_called()
        self.app.sources['test-twitter'].run.assert_called()
        self.app.statedb.save_state.assert_called()
        # should run 4 times, sources*operators.
        self.assertEqual(self.app.sources['test-twitter'].process.call_count, 4)



import unittest
from unittest.mock import patch
import os

import httpretty

import threatingestor.sources.git


class TestGit(unittest.TestCase):

    def setUp(self):
        self.git = threatingestor.sources.git.Plugin('mygit', 'https://example.mock/user/repo.git', '/tmp/test/repo')

    @patch('subprocess.check_output')
    def test_internal_git_cmd_calls_subprocess_git(self, check_output):
        threatingestor.sources.git._git_cmd(['test'])
        check_output.assert_called_once_with(['git', 'test'])

    @patch('subprocess.check_output')
    @patch('os.chdir')
    def test_internal_git_cmd_chdir_manages_working_dir(self, chdir, check_output):
        current_dir = os.getcwd()
        threatingestor.sources.git._git_cmd_chdir('/test', ['test'])

        chdir.assert_any_call('/test')
        chdir.assert_called_with(current_dir)
        check_output.assert_called_once_with(['git', 'test'])

    @patch('threatingestor.sources.git._git_pull')
    @patch('threatingestor.sources.git._git_latest_hash')
    @patch('threatingestor.sources.git._git_diff_names')
    @patch('threatingestor.sources.git.io.open')
    def test_run_with_saved_state_pulls_diffs_parses(self, m_open, git_diff_names, git_latest_hash, git_pull):
        git_latest_hash.return_value = 'test_latest'
        git_diff_names.return_value = 'test_file\ntest_file.rule'
        m_open.return_value.__enter__.return_value.read.return_value = 'rule testrule { condition: false }'
        
        saved_state, artifact_list = self.git.run('test_saved')

        git_pull.assert_called_once_with(self.git.local_path)
        git_latest_hash.assert_called_once_with(self.git.local_path)
        git_diff_names.assert_called_once_with(self.git.local_path, 'test_saved')
        m_open.assert_called_once_with(os.path.join(self.git.local_path, 'test_file.rule'), 'r', encoding='utf-8', errors='ignore')
        self.assertEqual(saved_state, 'test_latest')
        self.assertIn('rule testrule { condition: false }', [str(x) for x in artifact_list])

    @patch('threatingestor.sources.git._git_clone')
    @patch('threatingestor.sources.git._git_latest_hash')
    @patch('threatingestor.sources.git._git_ls_files')
    @patch('threatingestor.sources.git.io.open')
    def test_run_without_saved_state_clones_lists_parses(self, m_open, git_ls_files, git_latest_hash, git_clone):
        git_latest_hash.return_value = 'test_latest'
        git_ls_files.return_value = 'test_file\ntest_file.yar'
        m_open.return_value.__enter__.return_value.read.return_value = 'rule testrule { condition: false }'
        
        saved_state, artifact_list = self.git.run(None)

        git_clone.assert_called_once_with(self.git.url, self.git.local_path)
        git_latest_hash.assert_called_once_with(self.git.local_path)
        git_ls_files.assert_called_once_with(self.git.local_path)
        m_open.assert_called_once_with(os.path.join(self.git.local_path, 'test_file.yar'), 'r', encoding='utf-8', errors='ignore')
        self.assertEqual(saved_state, 'test_latest')
        self.assertIn('rule testrule { condition: false }', [str(x) for x in artifact_list])

    @patch('threatingestor.sources.git._git_pull')
    @patch('threatingestor.sources.git._git_latest_hash')
    @patch('threatingestor.sources.git._git_diff_names')
    def test_run_with_saved_state_returns_early_if_nothing_new(self, git_diff_names, git_latest_hash, git_pull):
        git_latest_hash.return_value = 'test_saved'
        git_diff_names.return_value = ''
        
        saved_state, artifact_list = self.git.run('test_saved')

        git_pull.assert_called_once_with(self.git.local_path)
        git_latest_hash.assert_called_once_with(self.git.local_path)
        git_diff_names.assert_called_once_with(self.git.local_path, 'test_saved')
        self.assertEqual(saved_state, 'test_saved')
        self.assertEqual(len(artifact_list), 0)




import unittest

import threatingestor.operators.sqlite
import threatingestor.artifacts


class TestThreatSQLite(unittest.TestCase):

    def setUp(self):
        # Set up an in-memory SQL database, we're doing full integration tests here.
        self.sqlite = threatingestor.operators.sqlite.Plugin(':memory:')

    def test_create_tables_creates_tables_for_each_artifact_type(self):
        # Since _create_tables was called by the constructor, just assert that
        # all the expected tables exist and have a sane structure.
        self.sqlite.cursor.execute('SELECT count() FROM sqlite_master WHERE type="table"')
        table_count = self.sqlite.cursor.fetchone()
        self.assertEqual((len(self.sqlite.artifact_types),), table_count)

        for artifact_type in self.sqlite.artifact_types:
            type_name = artifact_type.__name__.lower()
            self.sqlite.cursor.execute('SELECT name FROM sqlite_master WHERE type="table" and name=?', (type_name,))
            table_name = self.sqlite.cursor.fetchone()
            self.assertEqual((type_name,), table_name)

        # Sanity test.
        self.sqlite.cursor.execute('SELECT name FROM sqlite_master WHERE type="table" and name="domain"')
        table_name = self.sqlite.cursor.fetchone()
        self.assertEqual(("domain",), table_name)

    def test_handle_artifact_inserts_or_ignores(self):
        # First insert.
        self.sqlite.handle_artifact(threatingestor.artifacts.Domain(
            'test.com', 'name', reference_link='link', reference_text='text'
        ))
        query = 'SELECT artifact, reference_link, reference_text FROM domain WHERE artifact="test.com"'
        self.sqlite.cursor.execute(query)
        data = self.sqlite.cursor.fetchone()
        self.assertEqual(('test.com', 'link', 'text'), data)

        # Subsequent inserts are ignored.
        self.sqlite.handle_artifact(threatingestor.artifacts.Domain(
            'test.com', 'name', reference_link='different link', reference_text='more text'
        ))
        query = 'SELECT count() FROM domain WHERE artifact="test.com"'
        self.sqlite.cursor.execute(query)
        count = self.sqlite.cursor.fetchone()
        self.assertEqual((1,), count)

        query = 'SELECT artifact, reference_link, reference_text FROM domain WHERE artifact="test.com"'
        self.sqlite.cursor.execute(query)
        data = self.sqlite.cursor.fetchone()
        self.assertEqual(('test.com', 'link', 'text'), data)

    def test_artifact_types_are_set_if_passed_in_else_default(self):
        artifact_types = [threatingestor.artifacts.IPAddress, threatingestor.artifacts.URL]
        self.assertEqual(threatingestor.operators.sqlite.Plugin(':memory:', artifact_types=artifact_types).artifact_types, artifact_types)
        self.assertEqual(threatingestor.operators.sqlite.Plugin(':memory:').artifact_types, [
            threatingestor.artifacts.Domain,
            threatingestor.artifacts.Hash,
            threatingestor.artifacts.IPAddress,
            threatingestor.artifacts.URL,
            threatingestor.artifacts.YARASignature,
            threatingestor.artifacts.Task
        ])

    def test_filter_string_and_allowed_sources_are_set_if_passed_in(self):
        self.assertEqual(threatingestor.operators.sqlite.Plugin(':memory:', filter_string='test').filter_string, 'test')
        self.assertEqual(threatingestor.operators.sqlite.Plugin(':memory:', allowed_sources=['test-one']).allowed_sources, ['test-one'])



import requests

from threatingestor.sources import Source

class Plugin(Source):
    """VirusTotal Comments Source Plugin"""
    def __init__(self, name, user, api_key, limit=10):
        self.name = name
        self.user = user
        self.api_key = api_key
        self.limit = limit

    def run(self, saved_state):
        """
        Returns a list of artifacts and the saved state
        """

        headers = {
            "x-apikey": self.api_key,
            "accept": "application/json"
        }

        # Collects the 10 most recent comments from a specific user on VT
        response = requests.get(f"https://www.virustotal.com/api/v3/users/{self.user}/comments?limit={self.limit}", headers=headers)

        artifacts = []
        
        for comment in response.json()['data']:
            if saved_state and int(saved_state) <= int(comment['attributes']['date']):
                continue

            saved_state = comment['attributes']['date']
            artifacts += self.process_element(content=comment['attributes']['text'], reference_link=comment['links']['self'])

        return saved_state, artifacts



from __future__ import absolute_import

import twitter, requests
from loguru import logger

from threatingestor.sources import Source

class Plugin(Source):

    def __init__(self, name, api_key, api_secret_key, access_token, access_token_secret, bearer_token, defanged_only=True, **kwargs):
        self.name = name
        self.bearer_token = bearer_token

        """
        Depending on the endpoint being used, we need to modify the type of authentiation being used
        How to identity the type of authentication = https://developer.twitter.com/en/docs/authentication/overview
        """
        self.auth = {
            "OAuth": twitter.Twitter(auth=twitter.OAuth(token=access_token,
                                                        token_secret=access_token_secret,
                                                        consumer_key=api_key,
                                                        consumer_secret=api_secret_key),
                                                        api_version="1.1"),
            "OAuth2": twitter.Twitter(auth=twitter.OAuth2(bearer_token=self.bearer_token), api_version="2", format="")
        }

        # Let the user decide whether to include non-obfuscated URLs or not.
        self.include_nonobfuscated = not defanged_only

        # Forward kwargs.
        # NOTE: No validation is done here, so if the config is wrong, expect bad results.
        self.kwargs = kwargs
        self.endpoint = None

    def run(self, saved_state):
        # Modify kwargs to insert since_id.
        if saved_state:
            self.kwargs['since_id'] = saved_state

        artifacts = []

        def bearer_oauth2(r):
            r.headers['Authorization'] = f"Bearer {self.bearer_token}"
            r.headers['User-Agent'] = "InQuestThreatIngestor"
            return r

        if self.kwargs.get('list_id'):
            list_url = f"https://api.twitter.com/2/lists/{self.kwargs.get('list_id')}/tweets"
            self.endpoint = requests.get(url=list_url, auth=bearer_oauth2, params="tweet.fields=lang,author_id")

            for tweet in self.endpoint.json()['data']:
                artifacts += self.process_element(content=tweet['text'], reference_link=f"https://twitter.com/InQuest/status/{tweet['id']}", include_nonobfuscated=self.include_nonobfuscated)

        if self.kwargs.get('username') or self.kwargs.get('user_id'):
            try:
                # Get the id of the username
                user_id = self.kwargs.get('user_id') or self.auth['OAuth2'].users.by.username._username(_username=self.kwargs.get('username'))["data"]["id"]
                self.endpoint = self.auth['OAuth2'].users._id.tweets(_id=user_id)
            except twitter.api.TwitterHTTPError as e:
                # API error; log and return early
                logger.warning(f"Twitter API Error: {e}")
                return saved_state, []

            for tweet in self.endpoint['data']:
                saved_state = tweet['id']
                artifacts += self.process_element(content=tweet['text'], reference_link=f"https://twitter.com/InQuest/status/{tweet['id']}", include_nonobfuscated=self.include_nonobfuscated)
        
        if self.kwargs.get('query'):
            try:
                self.endpoint = self.auth['OAuth2'].tweets.search.recent(query=self.kwargs.get('query'))
            except twitter.api.TwitterHTTPError as e:
                # API error; log and return early
                logger.warning(f"Twitter API Error: {e}")
                return saved_state, []

            for tweet in self.endpoint['data']:
                saved_state = tweet['id']
                artifacts += self.process_element(content=tweet['text'], reference_link=f"https://twitter.com/InQuest/status/{tweet['id']}", include_nonobfuscated=self.include_nonobfuscated)

        # No endpoint specified, return early
        if not self.endpoint:
            return saved_state, []

        return saved_state, artifacts



import unittest

import threatingestor.operators
import threatingestor.artifacts


class TestOperators(unittest.TestCase):

    def test_default_artifact_types_is_empty(self):
        self.assertEqual(threatingestor.operators.Operator().artifact_types, [])

    def test_handle_artifact_raises_not_implemented(self):
        with self.assertRaises(NotImplementedError):
            threatingestor.operators.Operator().handle_artifact(None)

    def test_process_includes_only_artifact_types(self):
        threatingestor.operators.Operator.handle_artifact = lambda x, y: x.artifacts.append(y)
        operator = threatingestor.operators.Operator()
        operator.artifact_types = [threatingestor.artifacts.Domain]
        operator.artifacts = []

        artifact_list = [
            threatingestor.artifacts.IPAddress('21.21.21.21', '', ''),
            threatingestor.artifacts.Domain('test.com', '', ''),
            threatingestor.artifacts.URL('http://example.com', '', ''),
            threatingestor.artifacts.Domain('example.com', '', ''),
        ]

        operator.process(artifact_list)
        self.assertTrue(all([isinstance(x, threatingestor.artifacts.Domain) for x in operator.artifacts]))
        self.assertEqual(len(operator.artifacts), 2)

        operator.artifact_types = [threatingestor.artifacts.IPAddress, threatingestor.artifacts.URL]
        operator.artifacts = []
        operator.process(artifact_list)
        self.assertTrue(all([isinstance(x, threatingestor.artifacts.IPAddress) or isinstance(x, threatingestor.artifacts.URL) for x in operator.artifacts]))
        self.assertEqual(len(operator.artifacts), 2)

    def test_artifact_types_are_set_if_passed_in(self):
        artifact_types = [threatingestor.artifacts.IPAddress, threatingestor.artifacts.URL]
        self.assertEqual(threatingestor.operators.Operator(artifact_types=artifact_types).artifact_types, artifact_types)

    def test_process_includes_artifact_iff_filter_matches(self):
        threatingestor.operators.Operator.handle_artifact = lambda x, y: x.artifacts.append(y)
        operator = threatingestor.operators.Operator(filter_string='example.com')
        operator.artifact_types = [threatingestor.artifacts.Domain, threatingestor.artifacts.IPAddress, threatingestor.artifacts.URL]
        operator.artifacts = []

        artifact_list = [
            threatingestor.artifacts.IPAddress('21.21.21.21', '', ''),
            threatingestor.artifacts.Domain('test.com', '', ''),
            threatingestor.artifacts.URL('http://example.com', '', ''),
            threatingestor.artifacts.Domain('example.com', '', ''),
        ]

        operator.process(artifact_list)
        self.assertEqual(len(operator.artifacts), 2)
        self.assertNotIn(artifact_list[0], operator.artifacts)
        self.assertNotIn(artifact_list[1], operator.artifacts)
        self.assertIn(artifact_list[2], operator.artifacts)
        self.assertIn(artifact_list[3], operator.artifacts)

        operator.artifacts = []
        operator.filter_string = '21'
        operator.process(artifact_list)
        self.assertEqual(len(operator.artifacts), 1)
        self.assertIn(artifact_list[0], operator.artifacts)
        self.assertNotIn(artifact_list[1], operator.artifacts)
        self.assertNotIn(artifact_list[2], operator.artifacts)
        self.assertNotIn(artifact_list[3], operator.artifacts)

        operator.artifacts = []
        operator.filter_string = ''
        operator.process(artifact_list)
        self.assertEqual(len(operator.artifacts), 4)
        self.assertIn(artifact_list[0], operator.artifacts)
        self.assertIn(artifact_list[1], operator.artifacts)
        self.assertIn(artifact_list[2], operator.artifacts)
        self.assertIn(artifact_list[3], operator.artifacts)

        operator.artifacts = []
        operator.filter_string = 'is_domain'
        operator.process(artifact_list)
        self.assertEqual(len(operator.artifacts), 1)
        self.assertNotIn(artifact_list[0], operator.artifacts)
        self.assertNotIn(artifact_list[1], operator.artifacts)
        self.assertIn(artifact_list[2], operator.artifacts)
        self.assertNotIn(artifact_list[3], operator.artifacts)

    def test_process_includes_artifact_iff_source_name_is_allowed_or_allowed_is_empty(self):
        threatingestor.operators.Operator.handle_artifact = lambda x, y: x.artifacts.append(y)
        operator = threatingestor.operators.Operator()
        operator.artifact_types = [threatingestor.artifacts.Domain, threatingestor.artifacts.IPAddress, threatingestor.artifacts.URL]
        operator.artifacts = []

        artifact_list = [
            threatingestor.artifacts.IPAddress('21.21.21.21', 'source-1'),
            threatingestor.artifacts.Domain('test.com', 'source-1'),
            threatingestor.artifacts.URL('http://example.com', 'source-2'),
            threatingestor.artifacts.Domain('example.com', 'source-3'),
        ]

        operator.process(artifact_list)
        self.assertEqual(len(operator.artifacts), 4)
        self.assertIn(artifact_list[0], operator.artifacts)
        self.assertIn(artifact_list[1], operator.artifacts)
        self.assertIn(artifact_list[2], operator.artifacts)
        self.assertIn(artifact_list[3], operator.artifacts)

        operator.artifacts = []
        operator.allowed_sources = ['source-1']
        operator.process(artifact_list)
        self.assertEqual(len(operator.artifacts), 2)
        self.assertIn(artifact_list[0], operator.artifacts)
        self.assertIn(artifact_list[1], operator.artifacts)
        self.assertNotIn(artifact_list[2], operator.artifacts)
        self.assertNotIn(artifact_list[3], operator.artifacts)

        operator.artifacts = []
        operator.allowed_sources = ['source-2', 'source-3']
        operator.process(artifact_list)
        self.assertEqual(len(operator.artifacts), 2)
        self.assertNotIn(artifact_list[0], operator.artifacts)
        self.assertNotIn(artifact_list[1], operator.artifacts)
        self.assertIn(artifact_list[2], operator.artifacts)
        self.assertIn(artifact_list[3], operator.artifacts)

    def test_regex_allowed_sources(self):
        threatingestor.operators.Operator.handle_artifact = lambda x, y: x.artifacts.append(y)
        operator = threatingestor.operators.Operator(allowed_sources=['source-.*'])
        operator.artifact_types = [threatingestor.artifacts.Domain, threatingestor.artifacts.IPAddress, threatingestor.artifacts.URL]
        operator.artifacts = []

        artifact_list = [
            threatingestor.artifacts.IPAddress('21.21.21.21', 'source-1'),
            threatingestor.artifacts.Domain('test.com', 'source-1'),
            threatingestor.artifacts.URL('http://example.com', 'source-2'),
            threatingestor.artifacts.Domain('example.com', 'test-3'),
        ]

        operator.process(artifact_list)
        self.assertEqual(len(operator.artifacts), 3)
        self.assertIn(artifact_list[0], operator.artifacts)
        self.assertIn(artifact_list[1], operator.artifacts)
        self.assertIn(artifact_list[2], operator.artifacts)
        self.assertNotIn(artifact_list[3], operator.artifacts)



import feedparser
import regex as re

try:
    # feedparser 5.x
    from feedparser import _parse_date
except ImportError:
    # feedparser 6.x
    from feedparser.datetimes import _parse_date

import bs4

from threatingestor.sources import Source

class Plugin(Source):

    def __init__(self, name, url, feed_type, filter=None):
        self.name = name
        self.url = url
        self.feed_type = feed_type
        self.filter = filter

    def run(self, saved_state):
        feed = feedparser.parse(self.url)

        artifacts = []

        for item in list(reversed(feed['items'])):
            # Only new items
            published_parsed = item.get('published_parsed') or item.get('updated_parsed')

            if published_parsed and published_parsed <= _parse_date(saved_state or '0001-01-01'):
                continue

            try:
                soup = bs4.BeautifulSoup(item['content'][0]['value'], 'html.parser')
            except KeyError:
                try:
                    soup = bs4.BeautifulSoup(item['summary'], 'html.parser')
                except KeyError:
                    # Can't find any feed content, just skip this entry
                    continue

            # do some preprocessing to remove common obfuscation methods
            [x.unwrap() for x in soup.find_all('strong')]
            [x.unwrap() for x in soup.find_all('b')]
            [x.unwrap() for x in soup.find_all('em')]
            [x.unwrap() for x in soup.find_all('i')]
            soup = bs4.BeautifulSoup(soup.decode(), 'html.parser')

            text = ''

            if self.filter is not None:

                rss_query = re.compile(r"{0}".format(self.filter)).findall(str(self.filter.split('|')))

                for r in rss_query:
                    if self.feed_type == 'afterioc':
                        text = soup.get_text(separator=' ').split('Indicators of Compromise')[-1]

                        if r in text:
                            artifacts += self.process_element(text, item.get('link') or self.url, include_nonobfuscated=True)
                    elif self.feed_type == 'clean':
                        text = soup.get_text(separator=' ')

                        if r in text:
                            artifacts += self.process_element(text, item.get('link') or self.url, include_nonobfuscated=True)
                    else:
                        # Default: self.feed_type == 'messy'.
                        text = soup.get_text(separator=' ')
                        artifacts += self.process_element(text, item.get('link') or self.url)

            else:

                if self.feed_type == 'afterioc':
                    text = soup.get_text(separator=' ').split('Indicators of Compromise')[-1]
                    artifacts += self.process_element(text, item.get('link') or self.url, include_nonobfuscated=True)
                elif self.feed_type == 'clean':
                    text = soup.get_text(separator=' ')
                    artifacts += self.process_element(text, item.get('link') or self.url, include_nonobfuscated=True)
                else:
                    # Default: self.feed_type == 'messy'.
                    text = soup.get_text(separator=' ')
                    artifacts += self.process_element(text, item.get('link') or self.url)

            saved_state = item.get('published') or item.get('updated')

        return saved_state, artifacts



import re
import ipaddress
import iocextract

from urllib.parse import urlparse

class Artifact(object):
    """Artifact base class."""
    def __init__(self, artifact, source_name, reference_link=None, reference_text=None):
        self.artifact = artifact
        self.source_name = source_name
        self.reference_link = reference_link or ''
        self.reference_text = reference_text or ''


    def match(self, pattern):
        """Return True if regex pattern matches the deobfuscated artifact, else False.

        May be overridden or extended by child classes.
        """
        regex = re.compile(pattern)
        return True if regex.search(self.__str__()) else False


    def format_message(self, message, **kwargs):
        """Allow string interpolation with artifact contents.

        Optionally extend in child classes to add support for more
        specific interpolations.

        Supported variables:

        * {artifact}
        * {reference_text}
        * {reference_link}
        """
        return message.format(
            artifact=str(self),
            reference_text=self.reference_text,
            reference_link=self.reference_link,
            **kwargs
        )


    def _stringify(self):
        """Return str representation of the artifact.

        May be overridden in child classes.
        """
        return self.artifact


    def __str__(self):
        return self._stringify()


class URL(Artifact):
    """URL artifact abstraction, unicode-safe."""
    def _match_expression(self, pattern):
        """Process pattern as a condition expression.

        Raises ValueError if not a valid expression, else returns the
        truthiness of the expression.
        """
        NOT = 'not '
        conditions = [c.strip().lower() for c in pattern.split(',')]
        for condition in conditions:
            if condition.lstrip(NOT) not in URL.__dict__:
                raise ValueError('not a condition expression')
            else:
                result = URL.__dict__[condition.lstrip(NOT)](self)
                if condition.startswith(NOT) and result:
                    return False
                elif not condition.startswith(NOT) and not result:
                    return False
        return True


    def match(self, pattern):
        """Filter on some predefined conditions or a regex pattern.

        If pattern can be parsed as one of the conditions below, it returns the
        truthiness of the resulting expression; otherwise it is treated as regex.

        Valid conditions:

        * is_obfuscated
        * is_ipv4
        * is_ipv6
        * is_ip
        * is_domain
        * not {any above condition}
        * {any comma-separated list of above conditions}

        For example:

        * is_obfuscated, not is_ip
        * not is_obfuscated, is_domain
        """
        try:
            return self._match_expression(pattern)
        except ValueError:
            # not a valid condition expression, treat as regex instead
            return super(URL, self).match(pattern)


    def format_message(self, message, **kwargs):
        """Allow string interpolation with artifact contents.

        Supported variables:

        * {url}
        * {defanged}
        * {domain}
        * All supported variables from Artifact.format_message
        """
        return super(URL, self).format_message(message, url=str(self),
                                               domain=self.domain(),
                                               defanged=iocextract.defang_data(str(self)))


    def _stringify(self):
        """Always returns deobfuscated URL."""
        return iocextract.refang_data(self.artifact)


    def is_obfuscated(self):
        """Boolean: is an obfuscated URL?"""
        stringlike = self._stringify()
        if stringlike != self.artifact:
            # don't treat "example.com" as obfuscated
            if stringlike != 'http://' + self.artifact:
                return True
        return False


    def is_ipv4(self):
        """Boolean: URL network location is an IPv4 address, not a domain?"""
        parsed = urlparse(iocextract.refang_data(self.artifact))

        try:
            ipaddress.IPv4Address(parsed.netloc.split(':')[0].replace('[', '').replace(']', '').replace(',', '.'))
        except ValueError:
            return False

        return True


    def is_ipv6(self):
        """Boolean: URL network location is an IPv6 address, not a domain?"""
        # fix urlparse exception
        parsed = urlparse(iocextract.refang_data(self.artifact))

        # Handle RFC 2732 IPv6 URLs with and without port, as well as non-RFC IPv6 URLs
        if ']:' in parsed.netloc:
            ipv6 = ':'.join(parsed.netloc.split(':')[:-1])
        else:
            ipv6 = parsed.netloc

        try:
            ipaddress.IPv6Address(ipv6.replace('[', '').replace(']', ''))
        except ValueError:
            return False

        return True


    def is_ip(self):
        """Boolean: URL network location is an IP address, not a domain?"""
        return self.is_ipv4() or self.is_ipv6()


    def domain(self):
        """Deobfuscated domain; undefined behavior if self.is_ip()."""
        return urlparse(self._stringify()).netloc.split(':')[0]


    def is_domain(self):
        """Boolean: URL network location might be a valid domain?"""
        try:
            # can't have non-ascii
            self.domain().encode('ascii')
        except UnicodeEncodeError:
            return False
        return not self.is_ip() and len(self.domain()) > 3 and '.' in self.domain()[1:-1] and \
               all([x.isalnum() or x in '-.' for x in self.domain()]) and \
               self.domain()[self.domain().rfind('.')+1:].isalpha() and len(self.domain()[self.domain().rfind('.')+1:]) > 1


    def deobfuscated(self):
        """Named method for clarity, same as str(my_url_object)."""
        return self.__str__()


class IPAddress(Artifact):
    """IP address artifact abstraction.

    Use version and ipaddress() for processing.
    """
    def format_message(self, message, **kwargs):
        """Allow string interpolation with artifact contents.

        Supported variables:

        * {ipaddress}
        * {defanged}
        * All supported variables from Artifact.format_message
        """
        return super(IPAddress, self).format_message(message, ipaddress=str(self),
                                                     defanged=iocextract.defang_data(str(self)))


    def _stringify(self):
        """Always returns deobfuscated IP."""
        return self.artifact.replace('[', '').replace(']', '').split('/')[0].split(':')[0].split(' ')[0]


    @property
    def version(self):
        """Returns 4, 6, or None."""
        try:
            return ipaddress.IPv4Address(self._stringify()).version
        except ValueError:
            try:
                return ipaddress.IPv6Address(self._stringify()).version
            except ValueError:
                return None


    def ipaddress(self):
        """Return ipaddress.IPv4Address or ipaddress.IPv6Address object, or raise ValueError."""
        version = self.version
        if version == 4:
            return ipaddress.IPv4Address(self._stringify())
        elif version == 6:
            return ipaddress.IPv6Address(self._stringify())
        else:
            raise ValueError(u"Invalid IP address '{ip}'".format(ip=self.artifact))


class Domain(Artifact):
    """Domain artifact abstraction"""
    def format_message(self, message, **kwargs):
        """Allow string interpolation with artifact contents.

        Supported variables:

        * {domain}
        * {defanged}
        * All supported variables from Artifact.format_message
        """
        return super(Domain, self).format_message(message, domain=str(self),
                                                  defanged=iocextract.defang_data(str(self)))


class Hash(Artifact):
    """Hash artifact abstraction."""
    # Types
    MD5 = 'md5'
    SHA1 = 'sha1'
    SHA256 = 'sha256'
    SHA512 = 'sha512'


    def format_message(self, message, **kwargs):
        """Allow string interpolation with artifact contents.

        Supported variables:

        * {hash}
        * {hash_type}
        * All supported variables from Artifact.format_message
        """
        return super(Hash, self).format_message(message, hash=str(self),
                                                hash_type=self.hash_type() or 'hash')


    def hash_type(self):
        """Return the hash type as a string, or None."""
        if len(self.artifact) == 32:
            return self.MD5
        elif len(self.artifact) == 40:
            return self.SHA1
        elif len(self.artifact) == 64:
            return self.SHA256
        elif len(self.artifact) == 128:
            return self.SHA512
        else:
            return None


class YARASignature(Artifact):
    """YARA signature artifact abstraction."""
    def format_message(self, message, **kwargs):
        """Allow string interpolation with artifact contents.

        Supported variables:

        * {yarasignature}
        * All supported variables from Artifact.format_message
        """
        return super(YARASignature, self).format_message(message, yarasignature=str(self))

class Email(Artifact):
    """
    Email artifact abstraction.
    """

    def format_message(self, message, **kwargs):
        """
        Allow string interpolation with artifact contents.

        Supported variables:

        * {email}
        * All supported variables from Artifact.format_message
        """

        return super(Email, self).format_message(message, email=str(self))

class Task(Artifact):
    """Generic Task artifact abstraction."""
    def format_message(self, message, **kwargs):
        """Allow string interpolation with artifact contents.

        Supported variables:

        * {task}
        * All supported variables from Artifact.format_message
        """
        return super(Task, self).format_message(message, task=str(self))


# Define string mappings for artifact types.
# At the bottom because it uses the classes we just defined.
STRING_MAP = {
    'url': URL,
    'ipaddress': IPAddress,
    'domain': Domain,
    'hash': Hash,
    'yarasignature': YARASignature,
    'email': Email,
    'task': Task,
}



import os
import io
import subprocess
from loguru import logger

from threatingestor.sources import Source
from threatingestor.exceptions import DependencyError

try:
    subprocess.check_output('git')
except FileNotFoundError:
    raise DependencyError("System dependency Git required for Git source is not installed")
except subprocess.CalledProcessError:
    # Non-zero exit codes are fine.
    pass

YARA_FILE_EXTS = [
    '.rule',
    '.yar',
    '.yara',
    '.rules',
]

class Plugin(Source):

    def __init__(self, name, url, local_path):
        self.name = name
        self.url = url
        self.local_path = local_path


    def run(self, saved_state):
        # Initialize.
        new_hash = saved_state
        all_filenames = []

        if saved_state:
            # Pull and diff to get filenames.
            try:
                _git_pull(self.local_path)
                new_hash = _git_latest_hash(self.local_path)
                all_filenames = _git_diff_names(self.local_path, saved_state).splitlines()
            except (OSError, subprocess.CalledProcessError):
                logger.exception(f"Error with git pull from '{self.local_path}'")

        else:
            # First run, we try cloning the repo and look at all files.
            try:
                _git_clone(self.url, self.local_path)
                new_hash = _git_latest_hash(self.local_path)
                all_filenames = _git_ls_files(self.local_path).splitlines()

            except subprocess.CalledProcessError:
                # Clone failed, maybe the repo already exists?
                # If so, update the saved state so we can pull on the next run.
                try:
                    new_hash = _git_latest_hash(self.local_path)

                except (OSError, subprocess.CalledProcessError):
                    # That also failed, so this is just a generic error.
                    logger.exception(f"Error with git clone of '{self.url}' to '{self.local_path}'")

            except OSError:
                logger.exception(f"Error with git clone of '{self.url}' to '{self.local_path}'")

        # If not modified or any errors, return immediately.
        if saved_state == new_hash:
            return saved_state, []

        # Otherwise, process YARA files.
        artifact_list = []
        for filename in all_filenames:
            if any([filename.endswith(x) for x in YARA_FILE_EXTS]):
                try:
                    with io.open(os.path.join(self.local_path, filename), 'r', encoding='utf-8', errors='ignore') as f:
                        artifact_list += self.process_element(f.read(), self.url, include_nonobfuscated=True)
                except OSError:
                    # The file was most likely deleted.
                    pass

        return new_hash, artifact_list


def _git_cmd(args):
    """Call a git command.

    :raises: subprocess.CalledProcessError
    """
    git_cmdline = ['git'] + args
    return subprocess.check_output(git_cmdline).decode('utf-8')


def _git_cmd_chdir(path, args):
    cwd = os.getcwd()
    os.chdir(path)
    output = _git_cmd(args)
    os.chdir(cwd)
    return output


def _git_clone(remote, local_path):
    return _git_cmd(['clone', '--', remote, local_path])


def _git_latest_hash(local_path):
    return _git_cmd_chdir(local_path, ['rev-parse', 'HEAD']).strip()


def _git_pull(local_path):
    return _git_cmd_chdir(local_path, ['pull'])


def _git_diff_names(local_path, prev_hash):
    return _git_cmd_chdir(local_path, ['diff', '--name-only', prev_hash])


def _git_ls_files(local_path):
    return _git_cmd_chdir(local_path, ['ls-files'])



import os
from setuptools import setup, find_packages

with open("README.md", "r") as fh:
    README = fh.read()

# allow setup.py to be run from any path
os.chdir(os.path.normpath(os.path.join(os.path.abspath(__file__), os.pardir)))

requirements = open(os.path.join(os.path.dirname(__file__),
            'requirements.txt')).read()
requires = requirements.strip().split('\n')

extra_requirements = open(os.path.join(os.path.dirname(__file__),
            'requirements-testing.txt')).read()
extra_requires = extra_requirements.strip().split('\n')
if not isinstance(extra_requires, list):
    print("requirements-testing.txt was not parsed correctly.")
    extra_requires = []


setup(
    name='threatingestor',
    version='1.3.3',
    include_package_data=True,
    install_requires=requires,
    extras_require={
        'twitter': ['twitter'],
        'rss': ['feedparser'],
        'misp': ['PyMISP'],
        'threatkb': ['threatkb'],
        'beanstalk': ['greenstalk'],
        'sqs': ['boto3'],
        'mysql': ['pymysql'],
        'extras': ['hug', 'boto3', 'greenstalk', 'watchdog'],
        'all': extra_requires,
    },
    entry_points={
          'console_scripts': [
              'threatingestor = threatingestor:main'
          ]
    },
    license='GPL',
    description='Extract and aggregate IOCs from threat feeds.',
    long_description=README,
    long_description_content_type='text/markdown',
    url='https://github.com/InQuest/ThreatIngestor',
    author='InQuest Labs',
    author_email='labs@inquest.net',
    packages=find_packages(),
    classifiers=[
        'Intended Audience :: Developers',
        'Intended Audience :: Information Technology',
        'Intended Audience :: System Administrators',
        'License :: OSI Approved :: GNU General Public License v2 (GPLv2)',
        'Operating System :: OS Independent',
        'Programming Language :: Python :: 3.6',
        'Programming Language :: Python :: 3.7',
        'Programming Language :: Python :: 3 :: Only',
        'Topic :: Security',
        'Topic :: Internet',
    ],
)



import unittest

import ipaddress

import threatingestor.artifacts

class TestArtifacts(unittest.TestCase):
    
    def setUp(self):
        pass

    def test_unicode_support(self):
        self.assertEqual(type(threatingestor.artifacts.Artifact('test', 'test').__str__()), str)
        self.assertEqual(type(threatingestor.artifacts.Artifact(u't\u1111st', 'test').__str__()), str)

    def test_url_ipv4(self):
        self.assertTrue(threatingestor.artifacts.URL('http://192.168.0.1', '').is_ipv4())
        self.assertTrue(threatingestor.artifacts.URL('http://192,168,0,1', '').is_ipv4())
        self.assertTrue(threatingestor.artifacts.URL('http://192.168.0.1:80/path', '').is_ipv4())
        self.assertTrue(threatingestor.artifacts.URL('http://192[.]168[.]0[.]1:80/path', '').is_ipv4())
        self.assertTrue(threatingestor.artifacts.URL('192[.]168[.]0[.]1:80/path', '').is_ipv4())
        self.assertTrue(threatingestor.artifacts.URL('192.168.0.1', '').is_ipv4())
        self.assertTrue(threatingestor.artifacts.URL('tcp://192[.]168[.]0[.]1:80/path', '').is_ipv4())
        self.assertFalse(threatingestor.artifacts.URL('tcp://[fdc4:2581:575b:5a72:0000:0000:0000:0001]:80/path', '').is_ipv4())
        self.assertFalse(threatingestor.artifacts.URL('http://example.com:80/path', '').is_ipv4())

    def test_url_ipv6(self):
        self.assertTrue(threatingestor.artifacts.URL('http://fdc4:2581:575b:5a72:0000:0000:0000:0001', '').is_ipv6())
        self.assertTrue(threatingestor.artifacts.URL('http://[fdc4:2581:575b:5a72:0000:0000:0000:0001]:80/path', '').is_ipv6())
        self.assertTrue(threatingestor.artifacts.URL('[fdc4:2581:575b:5a72:0000:0000:0000:0001]:80/path', '').is_ipv6())
        self.assertFalse(threatingestor.artifacts.URL('fdc4:2581:575b:5a72:0000:0000:0000:0001', '').is_ipv6())
        self.assertTrue(threatingestor.artifacts.URL('tcp://[fdc4:2581:575b:5a72:0000:0000:0000:0001]:80/path', '').is_ipv6())
        self.assertFalse(threatingestor.artifacts.URL('tcp://192[.]168[.]0[.]1:80/path', '').is_ipv6())
        self.assertFalse(threatingestor.artifacts.URL('http://example.com:80/path', '').is_ipv6())

    def test_url_deobfuscation(self):
        self.assertEqual(threatingestor.artifacts.URL('http://example.com/', '').deobfuscated(), 'http://example.com/')
        self.assertEqual(threatingestor.artifacts.URL('http://example[.]com/', '').deobfuscated(), 'http://example.com/')
        self.assertEqual(threatingestor.artifacts.URL('http://example(.)com/', '').deobfuscated(), 'http://example.com/')
        self.assertEqual(threatingestor.artifacts.URL('hxxp://example[.]com/', '').deobfuscated(), 'http://example.com/')
        self.assertEqual(threatingestor.artifacts.URL('tcp://example[.]com/', '').deobfuscated(), 'http://example.com/')
        self.assertEqual(threatingestor.artifacts.URL('example[.]com/', '').deobfuscated(), 'http://example.com/')
        self.assertEqual(threatingestor.artifacts.URL('hxxp://example[dot]com/', '').deobfuscated(), 'http://example.com/')
        self.assertEqual(threatingestor.artifacts.URL('example[dot]com', '').deobfuscated(), 'http://example.com')
        self.assertEqual(threatingestor.artifacts.URL('192[.]168[dot]0[.]1', '').deobfuscated(), 'http://192.168.0.1')
        self.assertEqual(threatingestor.artifacts.URL('http://example[.com/', '').deobfuscated(), 'http://example.com/')
        self.assertEqual(threatingestor.artifacts.URL('http://example.com/ test', '').deobfuscated(), 'http://example.com/ test')
        self.assertEqual(threatingestor.artifacts.URL('http://[fdc4:2581:575b:5a72:0000:0000:0000:0001]:80/path', '').deobfuscated(),
                                        'http://[fdc4:2581:575b:5a72:0000:0000:0000:0001]:80/path')
        self.assertEqual(threatingestor.artifacts.URL('http://example.com[/]test', '').deobfuscated(), 'http://example.com/test')
        self.assertEqual(threatingestor.artifacts.URL('http://[example.com', '').deobfuscated(), 'http://example.com')
        self.assertEqual(threatingestor.artifacts.URL(u'http://example\u30fbcom', '').deobfuscated(), 'http://example.com')
        self.assertEqual(threatingestor.artifacts.URL('http://192,168,0,1', '').deobfuscated(), 'http://192.168.0.1')
        self.assertEqual(threatingestor.artifacts.URL('http://example,com/', '').deobfuscated(), 'http://example.com/')

    def test_is_obfuscated(self):
        self.assertFalse(threatingestor.artifacts.URL('example.com', '').is_obfuscated())
        self.assertFalse(threatingestor.artifacts.URL('http://example.com', '').is_obfuscated())
        self.assertFalse(threatingestor.artifacts.URL('http://example.com/[', '').is_obfuscated())
        self.assertFalse(threatingestor.artifacts.URL('http://192.168.0.1', '').is_obfuscated())
        self.assertFalse(threatingestor.artifacts.URL('192.168.0.1', '').is_obfuscated())
        self.assertTrue(threatingestor.artifacts.URL('hxxp://example.com', '').is_obfuscated())
        self.assertTrue(threatingestor.artifacts.URL('http://example[.]com', '').is_obfuscated())
        self.assertTrue(threatingestor.artifacts.URL('hxxp://example[.]com', '').is_obfuscated())
        self.assertTrue(threatingestor.artifacts.URL('example[.]com', '').is_obfuscated())
        self.assertTrue(threatingestor.artifacts.URL('http://example[dot]com', '').is_obfuscated())
        self.assertTrue(threatingestor.artifacts.URL('http://192[.]168[.]0[.]1', '').is_obfuscated())
        self.assertTrue(threatingestor.artifacts.URL('192[.]168[.]0[.]1', '').is_obfuscated())

    def test_is_domain(self):
        # valid
        self.assertTrue(threatingestor.artifacts.URL('example.com', '').is_domain())
        self.assertTrue(threatingestor.artifacts.URL('example,com', '').is_domain())
        self.assertTrue(threatingestor.artifacts.URL('http://example.com', '').is_domain())
        self.assertTrue(threatingestor.artifacts.URL('example[.]com', '').is_domain())
        self.assertTrue(threatingestor.artifacts.URL('http://example[dot]com', '').is_domain())
        self.assertTrue(threatingestor.artifacts.URL('http://exa-mple.com', '').is_domain())
        self.assertTrue(threatingestor.artifacts.URL('http://ex4mple.com', '').is_domain())
        self.assertTrue(threatingestor.artifacts.URL(u'http://example\u30fbcom', '').is_domain())
        self.assertTrue(threatingestor.artifacts.URL('short.is', '').is_domain())
        # invalid
        self.assertFalse(threatingestor.artifacts.URL('http://192[.]168[.]0[.]1', '').is_domain())
        self.assertFalse(threatingestor.artifacts.URL('192.168.0.1', '').is_domain())
        self.assertFalse(threatingestor.artifacts.URL('AAAAA', '').is_domain())
        self.assertFalse(threatingestor.artifacts.URL('+example.tld+', '').is_domain())
        self.assertFalse(threatingestor.artifacts.URL('85', '').is_domain())
        self.assertFalse(threatingestor.artifacts.URL('85.85', '').is_domain())
        self.assertFalse(threatingestor.artifacts.URL(u'exa\u30f2ple.com', '').is_domain())
        self.assertFalse(threatingestor.artifacts.URL('_____', '').is_domain())
        self.assertFalse(threatingestor.artifacts.URL('_____.tld', '').is_domain())
        self.assertFalse(threatingestor.artifacts.URL('http://example.com\\bad.doc', '').is_domain())
        self.assertFalse(threatingestor.artifacts.URL('x.x.x.x', '').is_domain())
        self.assertFalse(threatingestor.artifacts.URL('tooshor.t', '').is_domain())
        self.assertFalse(threatingestor.artifacts.URL('tooshor.', '').is_domain())
        self.assertFalse(threatingestor.artifacts.URL('http://123.123.123.123/test', '').is_domain())

    def test_url_domain_parsing(self):
        self.assertEqual(threatingestor.artifacts.URL('http://example.com/', '').domain(), 'example.com')
        self.assertEqual(threatingestor.artifacts.URL('http://example[.]com/', '').domain(), 'example.com')
        self.assertEqual(threatingestor.artifacts.URL('http://example[.]com/[.].com', '').domain(), 'example.com')
        self.assertEqual(threatingestor.artifacts.URL('tcp://example[.]com:80/', '').domain(), 'example.com')
        self.assertEqual(threatingestor.artifacts.URL('example[.]com/', '').domain(), 'example.com')

    def test_ipaddress_parsing(self):
        # str
        self.assertEqual(str(threatingestor.artifacts.IPAddress('192[.]168[.]0[.]1', '')), '192.168.0.1')
        self.assertEqual(str(threatingestor.artifacts.IPAddress('192.168.0.1/some/url', '')), '192.168.0.1')
        self.assertEqual(str(threatingestor.artifacts.IPAddress('192.168.0.1:9090', '')), '192.168.0.1')
        self.assertEqual(str(threatingestor.artifacts.IPAddress('192.168.0.1:9090/url', '')), '192.168.0.1')
        self.assertEqual(str(threatingestor.artifacts.IPAddress('192[.]168[.]0[.]1:9090/url', '')), '192.168.0.1')
        self.assertNotEquals(str(threatingestor.artifacts.IPAddress('192[.]188[dot]0[.]1:9090/url', '')), '192.168.0.1')
        # ipaddress
        self.assertEqual(threatingestor.artifacts.IPAddress('192[.]168[.]0[.]1', '').ipaddress(), ipaddress.IPv4Address(u'192.168.0.1'))
        self.assertEqual(threatingestor.artifacts.IPAddress('192.168.0.1/some/url', '').ipaddress(), ipaddress.IPv4Address(u'192.168.0.1'))
        self.assertEqual(threatingestor.artifacts.IPAddress('192.168.0.1:9090', '').ipaddress(), ipaddress.IPv4Address(u'192.168.0.1'))
        self.assertEqual(threatingestor.artifacts.IPAddress('192.168.0.1:9090/url', '').ipaddress(), ipaddress.IPv4Address(u'192.168.0.1'))
        self.assertEqual(threatingestor.artifacts.IPAddress('192[.]168[.]0[.]1:9090/url', '').ipaddress(), ipaddress.IPv4Address(u'192.168.0.1'))

    def test_match_artifact_regex(self):
        self.assertTrue(threatingestor.artifacts.IPAddress('192.168.0.1', '').match('192'))
        self.assertTrue(threatingestor.artifacts.IPAddress('192.168.0.1', '').match('168'))
        self.assertTrue(threatingestor.artifacts.IPAddress('192.168.0.1', '').match('192\.168'))
        self.assertTrue(threatingestor.artifacts.IPAddress('192.168.0.1', '').match('.'))
        self.assertTrue(threatingestor.artifacts.IPAddress('192.168.0.1', '').match('\.'))
        self.assertTrue(threatingestor.artifacts.IPAddress('192.168.0.1', '').match('.*'))
        self.assertTrue(threatingestor.artifacts.IPAddress('192[.]168.0[.]1', '').match('^192\.168\.0\.1$'))
        self.assertFalse(threatingestor.artifacts.IPAddress('192.168.0.1', '').match('168.192'))
        self.assertFalse(threatingestor.artifacts.IPAddress('192.168.0.1', '').match('192.*2'))
        self.assertFalse(threatingestor.artifacts.IPAddress('192.168.0.1', '').match('test'))
        self.assertTrue(threatingestor.artifacts.URL('http://example[.com', '').match('example.com'))
        self.assertTrue(threatingestor.artifacts.URL('http://example[.com/test.doc', '').match('.*\.doc'))
        # empty always matches
        self.assertTrue(threatingestor.artifacts.IPAddress('192.168.0.1', '').match(''))

    def test_match_url_condition_expression(self):
        # should be treated as regex
        self.assertTrue(threatingestor.artifacts.URL('http://example[.com/test.doc', '').match('.*\.doc'))
        self.assertTrue(threatingestor.artifacts.URL('http://example[.com/not is_ip', '').match('not is_ip.*'))
        self.assertTrue(threatingestor.artifacts.URL('http://example[.com/not is_ip,', '').match('not is_ip,.*'))
        # should be treated as condition
        self.assertTrue(threatingestor.artifacts.URL('http://example[.com/not is_ip', '').match('not is_ip'))
        self.assertTrue(threatingestor.artifacts.URL('http://example[.com/not is_ip', '').match('is_domain'))
        self.assertTrue(threatingestor.artifacts.URL('http://example[.com/', '').match('is_domain'))
        self.assertTrue(threatingestor.artifacts.URL('http://example[.com/', '').match('is_domain, not is_ip'))

        self.assertFalse(threatingestor.artifacts.URL('http://example[.com/', '').match('is_ip'))
        self.assertFalse(threatingestor.artifacts.URL('http://example[.com/', '').match('is_ip, not is_domain'))

        # first element true
        self.assertFalse(threatingestor.artifacts.URL('http://example[.com/', '').match('not is_ip, not is_domain'))
        # second element true
        self.assertFalse(threatingestor.artifacts.URL('http://example[.com/', '').match('is_ip, is_domain'))

        self.assertTrue(threatingestor.artifacts.URL('http://192.168[.]0.1/test.doc', '').match('is_ip, not is_domain, is_ipv4'))
        self.assertTrue(threatingestor.artifacts.URL('http://[[2001:db8:85a3::8a2e:370:7334]]/test.doc', '').match('is_ip, not is_domain, is_ipv6, not is_ipv4'))
        self.assertTrue(threatingestor.artifacts.URL('http://example[.com/test.doc', '').match('is_obfuscated'))
        self.assertFalse(threatingestor.artifacts.URL('http://example.com/test.doc', '').match('is_obfuscated'))
        self.assertTrue(threatingestor.artifacts.URL('http://example.com/test.doc', '').match('not is_obfuscated, is_domain'))

        # empty always matches
        self.assertTrue(threatingestor.artifacts.URL('http://example.com/test.doc', '').match(''))

    def test_second_arg_to_artifact_is_source_name(self):
        self.assertEqual(threatingestor.artifacts.URL('http://t.co/', 'a').source_name, 'a')
        self.assertEqual(threatingestor.artifacts.Domain('t.co', 'a').source_name, 'a')

    def test_hash_type_is_correct(self):
        self.assertEqual(threatingestor.artifacts.Hash('68b329da9893e34099c7d8ad5cb9c940', '').hash_type(), threatingestor.artifacts.Hash.MD5)
        self.assertEqual(threatingestor.artifacts.Hash('adc83b19e793491b1c6ea0fd8b46cd9f32e592fc', '').hash_type(), threatingestor.artifacts.Hash.SHA1)
        self.assertEqual(threatingestor.artifacts.Hash('01ba4719c80b6fe911b091a7c05124b64eeece964e09c058ef8f9805daca546b', '').hash_type(),
                          threatingestor.artifacts.Hash.SHA256)
        self.assertEqual(threatingestor.artifacts.Hash('be688838ca8686e5c90689bf2ab585cef1137c999b48c70b92f67a5c34dc15697b5d11c982ed6d71be1e1e7f7b4e0733884aa97c3f7a339a8ed03577cf74be09', '').hash_type(),
                          threatingestor.artifacts.Hash.SHA512)
        self.assertEqual(threatingestor.artifacts.Hash('adc83b191b1c6ea0fd8b46cd9f32e592fc', '').hash_type(), None)

    def test_artifact_format_message(self):
        artifact = threatingestor.artifacts.Artifact('test', '', 'link', 'text')
        message = '{artifact}: "{reference_text}" -- {reference_link}'
        expected = 'test: "text" -- link'
        self.assertEqual(artifact.format_message(message), expected)

    def test_url_format_message(self):
        artifact = threatingestor.artifacts.URL('http://example.com/', '', 'link', 'text')
        message = '{artifact}: "{reference_text}" -- {reference_link}'
        expected = 'http://example.com/: "text" -- link'
        self.assertEqual(artifact.format_message(message), expected)

        artifact = threatingestor.artifacts.URL('http://example.com/', '', 'link', 'text')
        message = '{url} ({domain})'
        expected = 'http://example.com/ (example.com)'
        self.assertEqual(artifact.format_message(message), expected)

    def test_domain_format_message(self):
        artifact = threatingestor.artifacts.Domain('example.com', '', 'link', 'text')
        message = '{artifact}: "{reference_text}" -- {reference_link}'
        expected = 'example.com: "text" -- link'
        self.assertEqual(artifact.format_message(message), expected)

        artifact = threatingestor.artifacts.Domain('example.com', '', 'link', 'text')
        message = '({domain})'
        expected = '(example.com)'
        self.assertEqual(artifact.format_message(message), expected)

    def test_ipaddress_format_message(self):
        artifact = threatingestor.artifacts.IPAddress('1.1.1.1', '', 'link', 'text')
        message = '{artifact}: "{reference_text}" -- {reference_link}'
        expected = '1.1.1.1: "text" -- link'
        self.assertEqual(artifact.format_message(message), expected)

        artifact = threatingestor.artifacts.IPAddress('1.1.1.1', '', 'link', 'text')
        message = '({ipaddress})'
        expected = '(1.1.1.1)'
        self.assertEqual(artifact.format_message(message), expected)

    def test_hash_format_message(self):
        artifact = threatingestor.artifacts.Hash('test', '', 'link', 'text')
        message = '{artifact}: "{reference_text}" -- {reference_link}'
        expected = 'test: "text" -- link'
        self.assertEqual(artifact.format_message(message), expected)

        artifact = threatingestor.artifacts.Hash('68b329da9893e34099c7d8ad5cb9c940', '', 'link', 'text')
        message = '{hash_type}: {hash}'
        expected = 'md5: 68b329da9893e34099c7d8ad5cb9c940'
        self.assertEqual(artifact.format_message(message), expected)

        artifact = threatingestor.artifacts.Hash('1231231', '', 'link', 'text')
        message = '{hash_type}: {hash}'
        expected = 'hash: 1231231'
        self.assertEqual(artifact.format_message(message), expected)

    def test_yarasignature_format_message(self):
        artifact = threatingestor.artifacts.YARASignature('test', '', 'link', 'text')
        message = '{artifact}: "{reference_text}" -- {reference_link}'
        expected = 'test: "text" -- link'
        self.assertEqual(artifact.format_message(message), expected)

        artifact = threatingestor.artifacts.YARASignature('test', '', 'link', 'text')
        message = '{yarasignature}'
        expected = 'test'
        self.assertEqual(artifact.format_message(message), expected)

    def test_task_format_message(self):
        artifact = threatingestor.artifacts.Task('test', '', 'link', 'text')
        message = '{artifact}: "{reference_text}" -- {reference_link}'
        expected = 'test: "text" -- link'
        self.assertEqual(artifact.format_message(message), expected)

        artifact = threatingestor.artifacts.Task('test', '', 'link', 'text')
        message = '{task}: {reference_link}'
        expected = 'test: link'
        self.assertEqual(artifact.format_message(message), expected)

    def test_defanged_format_message(self):
        artifact = threatingestor.artifacts.URL('http://example.com/', '', 'link', 'text')
        message = '{defanged}'
        expected = 'hxxp://example[.]com/'
        self.assertEqual(artifact.format_message(message), expected)

        artifact = threatingestor.artifacts.Domain('example.com', '', 'link', 'text')
        message = '{defanged}'
        expected = 'example[.]com'
        self.assertEqual(artifact.format_message(message), expected)

        artifact = threatingestor.artifacts.IPAddress('1.1.1.1', '', 'link', 'text')
        message = '{defanged}'
        expected = '1[.]1[.]1[.]1'
        self.assertEqual(artifact.format_message(message), expected)



import unittest
from unittest.mock import Mock, patch
import json

import threatingestor.operators.abstract_json
import threatingestor.artifacts


class TestAbstractJSON(unittest.TestCase):

    def setUp(self):
        self.abstract_json = threatingestor.operators.abstract_json.AbstractPlugin()

    def test_process_discards_ip_urls_if_filtered_out(self):
        # control
        self.abstract_json._put = Mock()
        self.abstract_json.filter_string = 'is_domain'
        self.abstract_json.handle_artifact(threatingestor.artifacts.URL('http://somedomain.com/test', '', ''))
        self.abstract_json._put.assert_called_once()

        # test
        self.abstract_json._put.reset_mock()
        self.abstract_json.process([threatingestor.artifacts.URL('http://123.123.123.123/test', '', '')])
        self.abstract_json._put.assert_not_called()

    def test_handle_artifact_passes_kwargs_url(self):
        self.abstract_json._put = Mock()
        self.abstract_json.kwargs = {
            'test_arg': 'test_val',
            'test_domain': '{domain}',
            'test_url': '{url}',
        }
        expected_content = {
            'test_arg': 'test_val',
            'test_domain': 'somedomain.com',
            'test_url': 'http://somedomain.com/test',
        }
        self.abstract_json.handle_artifact(threatingestor.artifacts.URL('http://somedomain.com/test', '', ''))
        self.abstract_json._put.assert_called_once_with(expected_content)

    def test_handle_artifact_passes_kwargs_hash(self):
        self.abstract_json._put = Mock()
        self.abstract_json.kwargs = {
            'test_arg': 'test_val',
            'test_hash': '{hash}',
        }
        expected_content = {
            'test_arg': 'test_val',
            'test_hash': 'test',
        }
        self.abstract_json.handle_artifact(threatingestor.artifacts.Hash('test', '', ''))
        self.abstract_json._put.assert_called_once_with(expected_content)

    def test_handle_artifact_passes_kwargs_ipaddress(self):
        self.abstract_json._put = Mock()
        self.abstract_json.kwargs = {
            'test_arg': 'test_val',
            'test_ipaddress': '{ipaddress}',
        }
        expected_content = {
            'test_arg': 'test_val',
            'test_ipaddress': 'test',
        }
        self.abstract_json.handle_artifact(threatingestor.artifacts.IPAddress('test', '', ''))
        self.abstract_json._put.assert_called_once_with(expected_content)

    def test_handle_artifact_passes_kwargs_domain(self):
        self.abstract_json._put = Mock()
        self.abstract_json.kwargs = {
            'test_arg': 'test_val',
            'test_domain': '{domain}',
        }
        expected_content = {
            'test_arg': 'test_val',
            'test_domain': 'test',
        }
        self.abstract_json.handle_artifact(threatingestor.artifacts.Domain('test', '', ''))
        self.abstract_json._put.assert_called_once_with(expected_content)

    def test_handle_artifact_passes_kwargs_yarasignature(self):
        self.abstract_json._put = Mock()
        self.abstract_json.kwargs = {
            'test_arg': 'test_val',
            'test_yarasignature': '{yarasignature}',
        }
        expected_content = {
            'test_arg': 'test_val',
            'test_yarasignature': 'test',
        }
        self.abstract_json.handle_artifact(threatingestor.artifacts.YARASignature('test', '', ''))

        self.abstract_json._put.assert_called_once_with(expected_content)

    def test_artifact_types_are_set_if_passed_in_else_default(self):
        artifact_types = [threatingestor.artifacts.IPAddress, threatingestor.artifacts.URL]

        self.assertEqual(threatingestor.operators.abstract_json.AbstractPlugin( artifact_types=artifact_types).artifact_types, artifact_types)
        self.assertEqual(threatingestor.operators.abstract_json.AbstractPlugin().artifact_types, [threatingestor.artifacts.URL])
        
    def test_init_sets_config_args(self):
        operator = threatingestor.operators.abstract_json.AbstractPlugin( filter_string='test', allowed_sources=['test-one'])
        self.assertEqual(operator.filter_string, 'test')
        self.assertEqual(operator.allowed_sources, ['test-one'])
